import numpy as np


def dft_phase(x: np.array, y: np.array, freq: float) -> float:
    """Calculates the phase from the discrete fourier transform. Useful for some things.

    Args:
        x (np.array): Input time
        y (np.array): Input flux
        freq (float): Single frequency at which to calculate phase

    Returns:
        float: Output phase
    """
    expo = 2.0 * np.pi * freq * x
    ft_real = np.sum(y * np.cos(expo))
    ft_imag = np.sum(y * np.sin(expo))
    return np.arctan2(ft_imag, ft_real)



import numpy as np
from astropy.timeseries import LombScargle


def amplitude_spectrum(
    t: np.array,
    y: np.array,
    fmin: float = None,
    fmax: float = None,
    oversample_factor: float = 5.0,
    freq: np.array = None,
) -> tuple:
    """Calculates the amplitude spectrum of input data.

    Args:
        t (np.array): Input times
        y (np.array): Input fluxes
        fmin (float, optional): Minimum frequency. Defaults to None.
        fmax (float, optional): Maximum frequency. Defaults to None.
        oversample_factor (float, optional): The oversampling rate. Defaults to 5.0.
        freq (np.array, optional): Optional custom frequency. Defaults to None.

    Returns:
        tuple: Frequency and amplitude
    """
    # t, y = self.time, self.residual
    if freq is None:
        tmax = t.max()
        tmin = t.min()
        df = 1.0 / (tmax - tmin)

        if fmin is None:
            fmin = df
        if fmax is None:
            fmax = 0.5 / np.median(np.diff(t))  # *nyq_mult

        freq = np.arange(fmin, fmax, df / oversample_factor)
    model = LombScargle(t, y)
    sc = model.power(freq, method="fast", normalization="psd")

    fct = np.sqrt(4.0 / len(t))
    amp = np.sqrt(sc) * fct

    return freq, amp


def psd(
    t: np.array,
    y: np.array,
    fmin: float = None,
    fmax: float = None,
    oversample_factor: int = 1,
) -> tuple:
    """Calculates the power spectral density spectrum of input data.
    Note that the time is assumed to be in `days'. The output frequency is converted to uHz.

    Args:
        t (np.array): Input times
        y (np.array): Input fluxes
        fmin (float, optional): Minimum frequency. Defaults to None.
        fmax (float, optional): Maximum frequency. Defaults to None.
        oversample_factor (float, optional): The oversampling rate. Defaults to 5.0.

    Returns:
        tuple: Frequency and power spectral density
    """
    tmax = t.max()
    tmin = t.min()
    df = 1.0 / (tmax - tmin)
    if fmin is None:
        fmin = df
    if fmax is None:
        fmax = 0.5 / np.median(np.diff(t))  # *nyq_mult

    N = len(t)
    freq = np.arange(fmin, fmax, df / oversample_factor)

    nu = 0.5 * (fmin + fmax)
    freq_window = np.arange(fmin, fmax, df / oversample_factor)
    power_window = (
        LombScargle(t, np.sin(2 * np.pi * nu * t)).power(
            freq_window, normalization="psd"
        )
        / N
        * 4.0
    )
    Tobs = 1.0 / np.sum(np.median(freq_window[1:] - freq_window[:-1]) * power_window)
    p = (LombScargle(t, y).power(freq, normalization="psd") / N * 4.0) * Tobs

    return freq, p


def power_spectrum(*args, **kwargs) -> tuple:
    """A simple wrapper to return the power spectrum

    Returns:
        tuple: Frequency and power
    """
    freq, amp = amplitude_spectrum(*args, **kwargs)
    return freq, amp**2


def psd_muHz(t, y, **kwargs) -> tuple:
    freq, p = psd(t, y * 1e6, **kwargs)

    freq = freq / (24 * 3600) * 1e6  # c/d to muHz
    p = p * (24 * 3600) * 1e-6  #  # ppm^2/(c/d) to ppm^2/muHz
    return freq, p



import tqdm
import numpy as np
import matplotlib.pyplot as plt

from .periodogram import amplitude_spectrum


def gauss(x, A, x0, sigma):
    return A * np.exp(-((x - x0) ** 2) / (2 * sigma**2))


def sliding_window(
    time, flux, fmin, fmax, segment_size=5, gwidth=10, plot=True, cmap="Blues"
):
    t0s = np.arange(time.min(), time.max(), segment_size)
    amps = []
    for i in tqdm.tqdm(range(len(t0s[:-1]))):
        g = gauss(time, 1, t0s[i], gwidth)
        # g = boxcar(time, t0s[i], gwidth)
        f, a = amplitude_spectrum(time, flux * g, fmin=fmin, fmax=fmax)
        amps.append(a)

    if plot:
        fig, axes = plt.subplots(
            2, 2, gridspec_kw={"height_ratios": [0.5, 1], "width_ratios": [1, 0.5]}
        )
        axes = axes.flatten()

        ax = axes[0]
        ax.plot(time, flux, c="k", lw=0.8)
        ax.set(xlim=[time.min(), time.max()], xticks=[], ylabel="Flux")

        ax = axes[1]
        ax.set_axis_off()

        ax = axes[2]
        ax.imshow(
            np.sqrt(amps).T,
            aspect="auto",
            extent=[t0s.min(), t0s.max(), fmin, fmax],
            origin="lower",
            cmap=cmap,
        )
        ax.set(xlabel="Time [day]", ylabel=r"Frequency [$d^{-1}$]")

        ax = axes[3]
        freq, amplitude = amplitude_spectrum(time, flux, fmin, fmax)
        ax.plot(amplitude, freq, c="k", lw=0.7)
        ax.set(xlim=[0, None], ylim=[fmin, fmax], yticks=[], xlabel="Power")
        ax.set_xticks(ax.get_xticks()[1:])
        plt.subplots_adjust(hspace=0.02, wspace=0.02)
        plt.show()
    return t0s, amps


def sliding_window_box(time, flux, fmin, fmax, segment_size=5, plot=True, cmap="Blues"):
    t0s = np.arange(time.min(), time.max(), segment_size)

    df = 1.0 / (time.max() - time.min())
    freq = np.arange(fmin, fmax, df / 5)
    amps = np.empty((len(t0s) - 1, len(freq)))
    amps[:] = np.nan
    for i, t0 in tqdm.tqdm(enumerate(t0s[:-1]), total=len(t0s) - 1):
        m = (t0 <= time) & (time < t0s[i + 1])
        if m.sum() < 100:
            continue
        f, a = amplitude_spectrum(time[m], flux[m], freq=freq)
        amps[i] = a
    if plot:
        fig, axes = plt.subplots(
            2,
            2,
            figsize=[10, 4],
            gridspec_kw={"height_ratios": [0.5, 1], "width_ratios": [1, 0.5]},
        )
        axes = axes.flatten()

        ax = axes[0]
        ax.plot(time, flux, c="k", lw=0.8)
        ax.set(xlim=[time.min(), time.max()], xticks=[], ylabel="Flux")

        ax = axes[1]
        ax.set_axis_off()

        ax = axes[2]
        ax.imshow(
            np.array(amps).T,
            aspect="auto",
            extent=[t0s.min(), t0s.max(), fmin, fmax],
            origin="lower",
            cmap=cmap,
            # interpolation='none'
        )
        ax.set(xlabel="Time [day]", ylabel=r"Frequency [$d^{-1}$]")

        ax = axes[3]
        freq, amplitude = amplitude_spectrum(time, flux, fmin, fmax)
        ax.plot(amplitude, freq, c="k", lw=0.7)
        ax.set(xlim=[0, None], ylim=[fmin, fmax], yticks=[], xlabel="Power")
        ax.set_xticks(ax.get_xticks()[1:])
        plt.subplots_adjust(hspace=0.02, wspace=0.02)
        plt.show()
    return t0s, amps



from __future__ import division, print_function

import numpy as np
import matplotlib.pyplot as plt

from .normalization import amplitude_spectrum, psd_muHz, power_spectrum


class Periodogram:
    """The big kahuna."""

    def __init__(self, frequency, power, meta={}):
        self.frequency = frequency
        self.power = power

        self.meta = meta

    @staticmethod
    def calculate(t: np.array, y: np.array, normalization: str = None, **kwargs):
        """Calculates the periodogram given the normalization. The normalization
        must be either one of `amplitude`, `power`, or `psd`. Note that if choosing
        `psd`, the periodogram will automatically convert the units such that the
        resulting spectrum is in microhertz and ppm^2/microhertz.

        Args:
            t (np.array): Input time values. Must be in units of days
            y (np.array): Input flux.
            normalization (str, optional): Normalization to use. Defaults to None.

        Returns:
            Periodogram: The normalized periodogram
        """
        normalization = normalization.lower()
        if normalization not in ["amplitude", "psd", "power"]:
            raise ValueError("Normalization must be one of `psd`, `amplitude`, `power")

        if normalization == "amplitude":
            return Periodogram(
                *amplitude_spectrum(t, y, **kwargs), meta={"normalization": "Amplitude"}
            )
        elif normalization == "psd":
            return Periodogram(*psd_muHz(t, y, **kwargs), meta={"normalization": "PSD"})
        elif normalization == "power":
            return Periodogram(
                *power_spectrum(t, y, **kwargs), meta={"normalization": "Power"}
            )

    def plot(self, ax=None):
        """Plots the periodogram

        Args:
            ax (_type_, optional): Matplotlib axis. Defaults to None.

        Returns:
            _type_: The plot
        """
        if ax is None:
            fig, ax = plt.subplots()
        ax.plot(self.frequency, self.power, lw=0.7, c="k")
        ax.set(xlim=[self.frequency[0], self.frequency[-1]])
        ax.set_xlabel(r"Frequency [day$^{-1}$]")
        if self.meta["normalization"] == "PSD":
            ax.set(
                xlabel=r"Frequency [$\mu$Hz]",
                ylabel=r"PSD [ppm$^2$ / $\mu$Hz]",
                xscale="log",
                yscale="log",
            )
        elif self.meta["normalization"] == "Amplitude":
            ax.set_ylabel("Amplitude")
            ax.set_ylim(0, None)
        elif self.meta["normalization"] == "Power":
            ax.set_ylabel("Power")
            ax.set_ylim(0, None)
        return ax

    def estimate_background(self, log_width=0.01):
        """Estimates the background noise level in the periodogram using a moving log filter

        Args:
            log_width (float, optional): Width of the filter. Defaults to 0.01.

        Returns:
            np.array: Array of backgrounds
        """
        x, y = self.frequency, self.power
        count = np.zeros(len(x), dtype=int)
        bkg = np.zeros_like(x)
        x0 = np.log10(x[0])
        while x0 < np.log10(x[-1]):
            m = np.abs(np.log10(x) - x0) < log_width
            bkg[m] += np.median(y[m])
            count[m] += 1
            x0 += 0.5 * log_width
        return bkg / count

    # def flatten(self, **kwargs):
    #     bkg = self.estimate_background(**kwargs)
    #     return Periodogram(self.frequency, self.power / bkg, meta=self.meta)


# class AmplitudeSpectrum(Periodogram):
#     """A subclass of Periodogram for amplitude normalizations

#     Args:
#         Periodogram (_type_): _description_
#     """

#     def __init__(self, *args, **kwargs):
#         super().__init__(*args, **kwargs)

#     def plot(self, *args, **kwargs):
#         ax = super().plot(*args, **kwargs)
#         ax.set_xlabel(r"Frequency [day$^{-1}$]")
#         ax.set_ylabel("Amplitude")
#         return ax


# class PowerSpectrum(Periodogram):
#     def __init__(self, *args, **kwargs):
#         super().__init__(*args, **kwargs)

#     def plot(self, *args, **kwargs):
#         ax = super().plot(*args, **kwargs)
#         ax.set_xlabel(r"Frequency [day$^{-1}$]")
#         ax.set_ylabel("Power")
#         return ax


# class PowerDensity(Periodogram):
#     def __init__(self, *args, **kwargs):
#         super().__init__(*args, **kwargs)

#     def plot(self, *args, **kwargs):
#         ax = super().plot(*args, **kwargs)
#         ax.set_xlabel(r"Frequency [$\mu$Hz]")
#         ax.set_ylabel(r"PSD [ppm$^2$ / $\mu$Hz]")
#         ax.set_xscale("log")
#         ax.set_yscale("log")
#         return ax



#!/usr/bin/env python
import os
import sys
from setuptools import setup

with open("requirements.txt") as f:
    install_requires = f.read().splitlines()

# Load the __version__ variable without importing the package already
exec(open("periodogram/version.py").read())

setup(
    name="periodogram",
    version=__version__,
    author="Daniel Hey",
    url="https://github.com/danhey/periodogram",
    packages=["periodogram"],
    description="Simple periodogram manipulation in Python.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
    ],
    install_requires=["astropy", "numpy", "matplotlib"],
)



from astropy.timeseries import LombScargle
from scipy import optimize
import numpy as np


def estimate_frequencies(
    x, y, fmin=None, fmax=None, max_peaks=3, oversample=4.0, optimize_freq=True
):
    """
    Attempts to pick out the best frequencies
    Parameters
    ----------
    x : float, optional
        Number of tuning steps for the sampler (default 3000)
    y : float, optional
        Number of samples from which to populate the trace (default 3000)
    fmin : bool, optional
        If set to True, the sampler will optimize the model before
        attempting to sample. If False (default), the sampler will
        initialise at the testpoints of your priors.
    fmax : float, optional
        The target acceptance ratio of the NUTS sampler (default 0.9).
    max_peaks : int
        Maximum number of frequencies to return (default 3)
    oversample : float
        Oversample factor for the spectrum (default 4.)
    optimize_freq : bool
        Whether to optimize the frequencies according to the
        Maelstrom model using Scipy.optimize (default True)
    Returns
    -------
    peaks : `numpy.ndarray`
        Array of frequencies of length `max_peaks`
    """
    tmax = x.max()
    tmin = x.min()
    dt = np.median(np.diff(x))
    df = 1.0 / (tmax - tmin)
    ny = 0.5 / dt

    if fmin is None:
        fmin = df
    if fmax is None:
        fmax = ny

    freq = np.arange(fmin, fmax, df / oversample)
    power = LombScargle(x, y).power(freq)

    # Find peaks
    peak_inds = (power[1:-1] > power[:-2]) & (power[1:-1] > power[2:])
    peak_inds = np.arange(1, len(power) - 1)[peak_inds]
    peak_inds = peak_inds[np.argsort(power[peak_inds])][::-1]
    peaks = []
    for j in range(max_peaks):
        i = peak_inds[0]
        freq0 = freq[i]
        alias = 2.0 * ny - freq0

        m = np.abs(freq[peak_inds] - alias) > 25 * df
        m &= np.abs(freq[peak_inds] - freq0) > 25 * df

        peak_inds = peak_inds[m]
        peaks.append(freq0)
    peaks = np.array(peaks)

    if optimize_freq:

        def chi2(nu):
            arg = 2 * np.pi * nu[None, :] * x[:, None]
            D = np.concatenate([np.cos(arg), np.sin(arg), np.ones((len(x), 1))], axis=1)

            # Solve for the amplitudes and phases of the oscillations
            DTD = np.matmul(D.T, D)
            DTy = np.matmul(D.T, y[:, None])
            w = np.linalg.solve(DTD, DTy)
            model = np.squeeze(np.matmul(D, w))

            chi2_val = np.sum(np.square(y - model))
            return chi2_val

        res = optimize.minimize(chi2, [peaks], method="L-BFGS-B")
        return res.x
    else:
        return peaks
